/* 
 * File:   CNFmanager.cpp
 * Author: Stanislav Pelák
 */

#include "CNFmanager.h"
#include <ctime>
using namespace std;

/**
 * Returns keys of map
 * @param m
 * @return 
 */
template <class T>
vector<string> CNFmanager::GetMapKeys(map<string, T>* m) {
    vector<string> v;
    for(typename map<string,T>::iterator it = m->begin(); it != m->end(); ++it) {
        v.push_back(it->first);
    }
    return v;
}

/**
 * Replaces variables in mask according to dictionary
 * @param mask
 * @param dictionary
 * @return string
 */
string CNFmanager::replaceVariables(string mask, map<string, int>* dictionary) {
    istringstream f(mask);
    ostringstream s;
    string line;    
    while (getline(f, line)) {
        string v;
        if(line.length() > 1) {
            s << (*dictionary)[line] << " ";
        } else if(line == "-") {
            s << "-";
        } else {
            s << "0 ";
        }
    }
    string m(s.str());
    return m;
}

/**
 * Concatenates clausules
 * @param keys
 * @param cnf
 * @return 
 */
string CNFmanager::concatClausules(vector<string>* keys, map<string, string>* cnf) {
    string result = "";
    for(int i = 0; i < keys->size(); ++i) {
        result += (*cnf)[(*keys)[i]];
    }
    return result;
}

string CNFmanager::xorMergeToDIMACS(Circuit* first, Circuit* second) {
    int lastIndex = 0;
    map<string, int> firstTrans;
    map<string, int> secondTrans;
    
    map<string, string> firstCNF = first->GetCNF();
    map<string, string> secondCNF = second->GetCNF();
    
    //mapping inputs from both circuits to same indices
    vector<string> firstInputs = first->getInputsKeys();
    vector<string> secondInputs = second->getInputsKeys();
    for(int i = 0; i < firstInputs.size(); ++i) {
        string in1 = "$" + firstInputs[i] + "$";
        firstTrans[in1] = ++lastIndex;
        
        string in2 = "$" + secondInputs[i] + "$";
        secondTrans[in2] = lastIndex;
    }
    
    //gates mapping to indices
    vector<string> firstKeys = CNFmanager::GetMapKeys(&firstCNF);
    for(int i = 0; i < firstKeys.size(); ++i) {
        string fk = "$" + firstKeys[i] + "$";
        firstTrans[fk] = ++lastIndex;
    }
    vector<string> secondKeys = CNFmanager::GetMapKeys(&secondCNF);
    for(int i = 0; i < secondKeys.size(); ++i) {
        string sk = "$" + secondKeys[i] + "$";
        secondTrans[sk] = ++lastIndex;
    }
    
    string tmp1 = CNFmanager::replaceVariables(CNFmanager::concatClausules(&firstKeys, &firstCNF), &firstTrans);
    string tmp2 = CNFmanager::replaceVariables(CNFmanager::concatClausules(&secondKeys, &secondCNF), &secondTrans);
    
    string dimacsClausules = tmp1 + tmp2;
    
    vector<int> xors;
    Gate* g = (GateMapper::getInstance())->getGate("XORA");
    string XORmask = g->GetCNFrep();
    for(int i = 0; i < first->getOutputs().size(); ++i) {
        map<string, int> XORmap;
        int variable = ++lastIndex;
        xors.push_back(variable);
        int out1 = firstTrans["$" + (first->getOutputs())[i]->GetID() + "$"];
        int out2 = secondTrans["$" + (second->getOutputs())[i]->GetID() + "$"];
        XORmap["$o$"] = variable;
        XORmap["$x$"] = out1;
        XORmap["$y$"] = out2;
        dimacsClausules += CNFmanager::replaceVariables(XORmask, &XORmap);
    }
    dimacsClausules += CNFmanager::joinWithOR(xors, ++lastIndex);
    int clauses = CNFmanager::countOccurences(dimacsClausules, " 0\n");
    ostringstream o;
    o << "p cnf " << lastIndex /*variables*/ << " " << clauses << endl << dimacsClausules;
    string dimacs(o.str());
    return dimacs;
}

string CNFmanager::joinWithOR(vector<int> gates, int ORindex) {
    ostringstream o;
    o << "-" << ORindex;
    for(int i = 0; i < gates.size(); ++i) {
        o << " " << gates[i];
    }
    o << " 0\n";
    for(int i = 0; i < gates.size(); ++i) {
        o << ORindex << " -" << gates[i] << " 0\n";
    }
    o << ORindex << " 0\n";
    string result(o.str());
    return result;
}

int CNFmanager::countOccurences(string str, string key) {
    string tmp = str;
    int cnt = 0;
    int pos = tmp.find(key);
    while(pos != string::npos) {
        ++cnt;
        tmp = tmp.substr(pos + 3, tmp.length() - pos - 3);
        pos = tmp.find(key);
    }
    return cnt;
}

