/* 
 * File:   Circuit.h
 * Author: Stanislav Pelák
 */

#ifndef CIRCUIT_H
#define	CIRCUIT_H

#include <map>
#include <string>
#include "Gene.h"
#include "Multilevel/Blif.h"
#include "GateMapper.h"
using std::map;
using std::string;

class Circuit {
public:
    Circuit(Blif* blif);
    Circuit(const Circuit& orig);
    virtual ~Circuit();
    
    Circuit* getWindow(int pos, int len, set<int> outputsIdxs);
    set<int> getWindowOutputsIdxs(int pos, int len);
    void applyChanges(int winStart, int winSize, Circuit* subcircuit, Circuit* referenceWindow);
    
    bool containsGene(string id);
    bool isEquivalent(Circuit* ref, bool fast);
    int countFitness(int maxGates);
    Blif* toBlif(vector<string> refOutputKeys);
    
    string getGenotype();
    map<string, string> GetCNF();
    vector<string> getUtilizedGenesIDs();
    vector<string> getInnerGenesIDs(bool strict);
    vector<string> getInputsKeys();
    vector<string> getOutputsKeys();
    
    vector<string> getGenesKeys() { return this->genesKeys; }
    vector<Gene*> getOutputs() const { return outputs; }
    string getModel() const { return model; }
    int getFitness() const { return fitness; }
    void setFitness(int fitness) { this->fitness = fitness; }
    int getInputsCnt() const { return inputsCnt; }
    int getGenesCnt() const { return genes.size(); };
    map<int, Gene*> getGenes() const { return genes; };
    
protected:
    
    string model;
    string genotype;
    int fitness;
    int inputsCnt;
    
    vector<string> inputsKeys;
    vector<string> outputsKeys;
    vector<string> genesKeys;
    
    //all genes (incl. inputs)
    map<int, Gene*> genes;
    vector<Gene*> outputs;
    
    Gene* getGeneById(string id);
    void copyGenotype(const Circuit& orig);
    void assignInputs(map<int, Gene*>* origGenes);
    
    Circuit(){};
    
private:
};

#endif	/* CIRCUIT_H */
