/* 
 * File:   Evolver.cpp
 * Author: Stanislav Pelák
 */

#include "Evolver.h"
#include "Generation.h"
#include "Configuration.h"
#include "OutputManager.h"
#include <ctime>

using namespace std;

void Evolver::evolveBasic(string path, string ofile, Circuit* referenceCircuit, int r, int c, int l, int lambda, int mutation, int inputs, int outputs, int pos) {
    if(Configuration::RANDOM_INDIVIDUALS) {
        cout << endl << "####################### Starting RANDOM CGP algorithm #######################" << endl << endl;
    } else {
        cout << endl << "####################### Starting basic CGP algorithm #######################" << endl << endl;
    }

    //init first generation
    Generation* act;
    if(Configuration::RANDOM_INDIVIDUALS) {
        act = new Generation(referenceCircuit, r, c, l, lambda, mutation, inputs, outputs);
    } else {
        act = new Generation(referenceCircuit, r, c, l, pos, lambda, mutation, inputs, outputs);
    }
    Generation* next;
    int max = 0;

    //evolution cycle
    cout << "---------------------------- EVOLVE ---------------------------" << endl;
    clock_t t;
    do {
        t = clock();
        next = act->evolve(true);
        t = clock() - t;
        int fit = Generation::getMaxFitness();
        if ((fit > max) || (act->getGenerationID() % 100 == 0))
            cout << "Generation: " << act->getGenerationID() << ", max fitness: " << fit << " (" << ((float) t) / CLOCKS_PER_SEC << "s)" << endl;
        max = fit;
        delete act;
        act = next;
    } while (true);

    //Cleanup
    cout << "---------------------------- CLEANUP ---------------------------" << endl;
    delete act;
    delete referenceCircuit;
    delete GateMapper::getInstance();
}
