/* 
 * File:   GateMapper.cpp
 * Author: Stanislav Pelák
 */

#include "GateMapper.h"
#include "Randomizer.h"
#include <iostream>
#include <ctime> 
#include <cstdlib>
#include <algorithm>
using namespace std;

map<int, Gate*> GateMapper::gates = GateMapper::fillMap();
GateMapper* GateMapper::instance = GateMapper::create();

/**
 * Initializes map of available gates and their CNF representation
 */
map<int, Gate*> GateMapper::fillMap() {
    map<int, Gate*> m;
    m[0] = new Gate("AND", 2, "-\n$x$\n-\n$y$\n$o$\n0\n$x$\n-\n$o$\n0\n$y$\n-\n$o$\n0\n");
    m[1] = new Gate("NAND", 2, "-\n$x$\n-\n$y$\n-\n$o$\n0\n$x$\n$o$\n0\n$y$\n$o$\n0\n");
    m[2] = new Gate("OR", 2, "$x$\n$y$\n-\n$o$\n0\n-\n$x$\n$o$\n0\n-\n$y$\n$o$\n0\n");
    m[3] = new Gate("NOR", 2, "$x$\n$y$\n$o$\n0\n-\n$x$\n-\n$o$\n0\n-\n$y$\n-\n$o$\n0\n");
    m[4] = new Gate("XORA", 2, "-\n$x$\n-\n$y$\n-\n$o$\n0\n-\n$x$\n$y$\n$o$\n0\n$x$\n-\n$y$\n$o$\n0\n$x$\n$y$\n-\n$o$\n0\n");
    m[5] = new Gate("XNORA", 2, "$x$\n$y$\n$o$\n0\n$x$\n-\n$y$\n-\n$o$\n0\n-\n$x$\n$y$\n-\n$o$\n0\n-\n$x$\n-\n$y$\n$o$\n0\n");
    m[6] = new Gate("BUF", 1, "-\n$x$\n$o$\n0\n$x$\n-\n$o$\n0\n");
    m[7] = new Gate("INV", 1, "-\n$x$\n-\n$o$\n0\n$x$\n$o$\n0\n");
    m[8] = new Gate("ZERO", 0, "-\n$o$\n0\n");
    m[9] = new Gate("ONE", 0, "$o$\n0\n");
    return m;
}

GateMapper* GateMapper::create() {
    return new GateMapper();
}

GateMapper::GateMapper() {
}

GateMapper::~GateMapper() {
    int gates = this->gates.size();
    for(int i = 0; i < gates; ++i)
        delete this->gates[i];
}

GateMapper* GateMapper::getInstance() {
    return GateMapper::instance;
}

int GateMapper::getCode(string name) const {
    for(int i = 0; i < gates.size(); ++i)
        if(gates[i]->GetName() == name)
            return i;
    return -1;
}

Gate* GateMapper::getGate(int code) const {
    return gates[code];
}

Gate* GateMapper::getGate(string name) const {
    std::transform(name.begin(), name.end(), name.begin(), ::toupper);
    for(int i = 0; i < gates.size(); ++i)
        if(gates[i]->GetName() == name)
            return gates[i];
    return NULL;
}

Gate* GateMapper::getRandomGate() const {
    int idx = this->getRandomGateCode();
    return this->gates[idx];
}

int GateMapper::getRandomGateCode() const {
    srand((unsigned) time(0) + Randomizer::getRandomizer()*31); 
    int random = rand();
    int idx = random % (this->gates.size());
    return idx;
}

