#ifndef bitvectorH
#define bitvectorH

#define DATATYPE unsigned				// size of a memory word - 32/64 bits
#define WORDLENGTH (sizeof(DATATYPE)*8)
#define SHIFT ((sizeof(DATATYPE) == 8 ? 6 : 5))			// 64 = 2^6 or 32 = 2^5
#define MASK DATATYPE(WORDLENGTH - 1)					// for modulo
#define ALL_ONE ~(DATATYPE)0				// all-1 vector

struct position {
	unsigned int index;
	unsigned int bit;
};

class bitvector
{
private:
	DATATYPE *data;
	unsigned int bitsize;		// size in bits

	inline void GetPosition( const unsigned int & pos, position & result ) const;

public:
	bool operator[] ( const unsigned int pos ) const;
	bitvector & operator= ( const bitvector &_vector );
	bool operator== ( const bitvector &_vector ) const;
	bitvector operator& ( const bitvector &_vector ) const;		// AND
	void operator&= ( const bitvector &_vector );				// AND
	bitvector operator| ( const bitvector &_vector ) const;		// OR
	void operator|= ( const bitvector &_vector );				// AND
	bitvector operator^ ( const bitvector &_vector ) const;		// XOR
	void operator^= ( const bitvector &_vector );				// AND
	bitvector operator~ () const;								// bitwise NOT
	
	bitvector(void);
	bitvector( const unsigned int _size );
	bitvector(const bitvector &_vector);
	~bitvector(void);

	inline unsigned int size(void) const { return bitsize; };						// returns the size in bits
	inline unsigned int items(void) const { return (bitsize == 0) ? 0 : ((bitsize-1) >> SHIFT)+1; };		// returns the size in datatype items
	void resize( const unsigned int _size );								// resizes the vector and fills new vector items with 0
	void resize( const unsigned int _size, const bool val );				// resizes the vector and assignes a value to new vector items
	void assign( const unsigned int _size, const bool val );				// assignes a value to all vector items
	void push_back( const bool bit );
	void setbit( const unsigned int pos, const bool val );
	void clear(void);
	bool empty(void) { return bitsize == 0; };
	bool contains( const bitvector &_vector ) const;		// returns true, if this contains _vector
	bool isNull(void) const;								// is the vector all-zero?
	bool isOne(void) const;									// is the vector all-one?
	DATATYPE GetWord( const unsigned int i ) const { return data[i]; };
};

#endif
