//---------------------------------------------------------------------------

#include "stdafx.h"

using namespace std;

#ifdef _MSC_VER
#pragma hdrstop
#endif

#include "utils.h"
#include "ctype.h"

//---------------------------------------------------------------------------

// PLA Parser

void GetLine( ifstream &f, char *line )
{
	f.getline(line, MAXLINE);
	if ( line[strlen(line)-1] == 13 ) line[strlen(line)-1] = 0;		// for Linux
	while ( line[strlen(line)-1] == '\\' ) {
		unsigned int i = strlen(line);
		f.getline(&line[strlen(line)-1], MAXLINE);
		if ( line[strlen(line)-1] == 13 ) line[strlen(line)-1] = 0;		// for Linux
	}
}

void GetLine( ifstream &f, string &line )
{
	char c;

	line.clear();
	do {						// skip trailing whitespaces
		f >> c;
	} while ( (c == ' ' || c == 9 || c == 10 || c == 13 || c == '\\') && !f.eof() );
	while ( c != 10 && c != 13 && !f.eof() ) {
		line.append(1, c);
		f.get(c);
		if ( c == '\\' ) 
			do {
				f.get(c);
			} while ( c == 10 || c == 13 );
	}
	line.append(1, 0);
}

string GetToken( const char *line, unsigned int &pos )
{
    string s;

    while ( line[pos] == ' ' || line[pos] == '|' || line[pos] == 10 || line[pos] == 9 || line[pos] == 13 ) pos++;
    while ( line[pos] != ' ' && line[pos] != '|' && line[pos] != 0 && line[pos] != 10 && line[pos] != 9 && line[pos] != 13 )
        s+=line[pos++];
    return s;
}

string GetToken( string &line, unsigned int &pos )
{
    string s;

    while ( line[pos] == ' ' || line[pos] == '|' || line[pos] == 10 || line[pos] == 9 || line[pos] == 13 ) pos++;
    while ( line[pos] != ' ' && line[pos] != '|' && line[pos] != 0 && line[pos] != 10 && line[pos] != 9 && line[pos] != 13 )
        s+=line[pos++];
    return s;
}

string RemoveSpaces( const string &s )
{
	unsigned int i, j;

	for ( i = 0; i < s.length(); i++ )
		if ( s[i] != ' ' && s[i] != 10 && s[i] != 13 && s[i] != 9 ) break;
	for ( j = s.length()-1; j >= i; j-- ) {
		if ( s[j] != ' ' && s[j] != 10 && s[j] != 13 && s[j] != 9 ) break;
	}
	return s.substr(i, j-i+1);
}

string GetExtension( const string &s )
// returns the filename extension
{
	string res;
	unsigned int i;

	i = s.find_last_of( "." );
	res = s.substr(i+1, s.length() );
	return res;
}

string GetFilename( const string &s )
// returns the filename from the complete path
{
	basic_string <char>::size_type j;
	string res;
	static const basic_string <char>::size_type npos = -1;

	j = s.rfind(".", s.size()-1);
	if ( j != npos ) res = s.substr(0, s.rfind(".", s.size()-1));
	else res = s;
	j = res.rfind("\\", res.size()-1);
	if ( j == npos ) j = res.rfind("/", res.size()-1);
	else if ( j == npos ) j = -1;
	j++;
	res = res.substr(j, res.size()-j);
	return res;
}

string GetPath( const string &s )
// returns the filename from the complete path
{
	basic_string <char>::size_type j;
	string res;
	static const basic_string <char>::size_type npos = -1;

	j = s.rfind("\\", s.size()-1);
	if ( j == npos )
		j = s.rfind("/", s.size()-1);
	if ( j == npos ) return res;
	res = s.substr(0, j+1);
	return res;
}

string int2bin( unsigned int num, unsigned int length )
{
    string res;
    int pos = 1;
	unsigned int i;

    for ( i = 0; i < length; i++ ) {
        res = (num & pos) ? "1" + res : "0" + res;
		pos <<= 1;
    }
    return res;
}

string bin2hex( const string num, unsigned int length )
{
    string res;
    int pos = 1;
	int i, j, k;
	unsigned int n;
	char c;

	j = 0;
	n = 0;
	k = length-1;
	res.assign(length, '0');
	for ( i = num.length()-1; i >= 0; i-- ) {
		n += ( 1 << j ) * ((num[i] == '1')?1:0);
		if ( j == 3 ) {
			if ( n < 10 ) c = n+'0';
			else c = n-10+'A';
			res[k--] = c;
			j = 0;
			n = 0;
		} else j++;
	}
	if ( j != 0 && n != 0) {
		if ( n < 10 ) c = n+'0';
		else c = n-10+'A';
		res[k--] = c;
		j = 0;
		n = 0;
	}
    return res;
}

string int2string( unsigned int num )
{
	char s[1000];

	sprintf(s, "%i", num);
	return string(s);
}

bool existsFile( string fn )
{
	ifstream f;

	f.open(fn.c_str());
	if ( f.is_open() ) {
		f.close();
		return true;
	} else return false;
}

void to_upper(std::string& s)
{
	string::iterator si;

	for ( si = s.begin(); si != s.end(); ++si )
    *si = toupper((unsigned char)*si);
}

string hex2bin( const string num )
{
	string res;
	unsigned int i, n;

	for ( i = 0; i < num.length(); i++ ) {
		if ( num[i] <= '9' ) n = num[i]-'0';
		else n = num[i]-'A'+10;
		res.append(int2bin(n, 4));
	}
	return res;
}
