//---------------------------------------------------------------------------

#ifndef BlifH
#define BlifH

#include "../Kernel/kernelincludes.h"

using std::string;
using std::vector;
using std::list;
                       
class Blif {
  private:
	string model;					// name of the model
	vector<string> inames;			// names of input ports. Must be defined
	vector<string> onames;			// names of output ports. Must be defined
        list<SoPLA> data;				    // here are the PLAs

	void clear(void);				// clears all the structure

  public:
	unsigned int CountNodeLevels( list<SoPLA>::iterator pla_it, unsigned int k, bool notriv );		// for GetNAND_NANDLevels

	Blif(void);
	Blif( const Blif *orig );			// non-copying constructor
	Blif( const vector<string> _inames, const vector<string> _onames );
    ~Blif(void);

	bool operator==( const Blif& blif );

	void LoadBlif( const char *fn );	// loads Blif from file
	void LoadBench( const char *fn );	// loads Blif from Bench file
	void SaveBlif( const char *fn );	// save as Blif
	void SaveBench( const char *fn );	// save as Bench

		// getters & setters
	unsigned int inputs(void) const;
	unsigned int outputs(void) const;
	list<SoPLA> *getData(void) { return &data; };
	const list<SoPLA> &getConstDataRef(void) const { return data; };
	string GetModel(void) const { return model; };
	void SetModel(const string _model) { model = _model; };
	unsigned int GetNodes(void) const { return data.size(); };
	vector<string> getINames(void) const { return inames;};
	vector<string> getONames(void) const { return onames;};
	string getIName( const unsigned int i ) const;
	string getOName( const unsigned int i ) const;

		// info
	unsigned int getLiterals(void);
	unsigned int getTerms(void) const;
	unsigned int getGEs(void);
	unsigned int GetLevels(bool notriv=false);		// returns the number of BLIF levels (longest path)
	double GetDelay(bool notriv=false);		// returns the delay. Uses SoPLA->delay
	double GetLEDelay(bool notriv=false);	// returns the delay using logical effort. Gate types are used to derive the delay
	unsigned int GetLevels_dummy(void);		// returns the number of BLIF levels (longest path). Dummy (tmp=0) gates are not considered
	unsigned int GetSize(void);						// returns the size. Uses SoPLA->size
	unsigned int GetNAND_NANDLevels(bool notriv) { return(GetNAND_NANDLevels(0, notriv)); };		// returns the number of levels of the NAND-NAND implementation
	unsigned int GetNAND_NANDLevels(unsigned int k, bool notriv);		// returns the number of levels of the NAND-NAND implementation for technology dependent synthesis
	unsigned int GetNonTrivNodes(void);	// returns the number of non-trivial (not inv, buff, const) nodes
	unsigned int CountBranchings(string sig);		// counts the number of signal branches
	unsigned int CountLEBranchings(string sig);		// counts the number of signal branches. For logical effort

	bool isPO(list<SoPLA>::iterator ni) const;			// is it a PO?

	unsigned int Difference( const Blif &blif ) const;		// count the difference between two Blifs, in terms of different gates

		// data manipulation
	void AppendPLA( SoPLA *_pla );
	void Splice( list<SoPLA> &plas );
	void Splice( SoPLA *_pla );
	void Splice( Blif &blif );
	void AppendBlif( Blif *_blif );		// Joins data from two Blifs. Input and output names are not considered!
	list<SoPLA>::iterator FindNode( const string &_name );		// Find a node by name. Returns data.end() if not found

};

//---------------------------------------------------------------------------
#endif
