/* 
 * File:   Circuit.h
 * Author: Stanislav Pelák
 */

#ifndef MUTABLE_CIRCUIT_H
#define	MUTABLE_CIRCUIT_H

#include "Circuit.h"

class MutableCircuit : public Circuit {
public:
    MutableCircuit(int nr, int nc, int l, int inputsCnt, int outputsCnt);
    MutableCircuit(const MutableCircuit& orig, bool copy);
    MutableCircuit(Circuit* orig, int rows, int cols, int l, int pos);
    MutableCircuit(Circuit* orig, int rows, int cols, int l, int pos, bool randomEquiv);
    ~MutableCircuit();
    
    static MutableCircuit* generateRandomEquivalent(Circuit* orig, int rows, int cols, int l, int pos);

    void mutate(int genes);
    
    int getColumns() const { return columns; }
    int getRows() const { return rows; }
    
private:
    int rows;
    int columns;
    int l;
    
    void initEmptyCircuit(int nr, int nc, int l, int inputsCnt, int outputsCnt);
    
    void generateCircuit(Circuit* c, int rows, int cols, int pos);
    void generateRandomGenotype(void);
    Gene* generateInput(int geneIndex);
    Gene* generateSafeInput(int geneIndex);
    void assignRandomOutput(int outputIdx);
    void fillGenes(string pref, int cnt, int offset, map<int, Gene*> &target, vector<string>* keys);
    
};


#endif	/* MUTABLE_CIRCUIT_H */

