/* 
 * File:   Gene.cpp
 * Author: Stanislav Pelák
 * 
 * Represents a single node in individual's phenotype.
 */

#include "Gene.h"
#include "Kernel/SoPLA.h"
#include "Circuit.h"
#include <iostream>
#include <algorithm>
#include <sstream>
using namespace std;

Gene::Gene(string id, int index) {
    this->id = id;
    this->gate = NULL;
    this->index = index;
    this->inPhenotype = false;
}

Gene::Gene(const Gene& orig) {
    this->id = orig.id;
    this->gate = orig.gate;
    this->index = orig.index;
    this->inPhenotype = orig.inPhenotype;
}

Gene::~Gene() {}

/**
 * Adds this gene's gate's CNF representation to map (recursively)
 */
void Gene::AppendGateCNF_rec(map<string, string>* cnf) {
    map<string, string>::iterator it = (*cnf).find(this->id);
    if((it != cnf->end()) || !this->gate)
        return;
    (*cnf)[this->id] = "0";
    
    string tmp = this->gate->GetCNFrep();
    int inputsCnt = this->inputs.size();
    Gene* tmpg;
    
    if(inputsCnt > 0) {
        tmpg = this->inputs[0];
        int pos = tmp.find("$x$");
        while(pos != string::npos) {
            tmp.replace(pos + 1, 1, tmpg->id);
            pos = tmp.find("$x$");
        }
        tmpg->AppendGateCNF_rec(cnf);
    }
    if(inputsCnt > 1) {
        tmpg = this->inputs[1];
        int pos = tmp.find("$y$");
        while(pos != string::npos) {
            tmp.replace(pos + 1, 1, tmpg->id);
            pos = tmp.find("$y$");
        }
        tmpg->AppendGateCNF_rec(cnf);
    }
    int pos = tmp.find("$o$");
    while(pos != string::npos) {
        tmp.replace(pos + 1, 1, this->id);
        pos = tmp.find("$o$");
    }
    (*cnf)[this->id] = tmp;
    return;
}

/**
 * Appends this gene's gate recursively (following gate's inputs)
 */
void Gene::AppendGateSoPLA_rec(map<string, SoPLA>* pla, vector<string>* keys) {
    map<string, SoPLA>::iterator it = (*pla).find(this->id);
    if((it != pla->end()) || !this->gate)
        return;
    SoPLA tmp;
    int inputs = this->inputs.size(); 
    (*pla)[this->id] = tmp;
    
    Gene* tmpg;
    for(int j = 0; j < inputs; ++j) {
        tmpg = this->inputs[j];
        tmpg->AppendGateSoPLA_rec(pla, keys);
    }
    
    ostringstream s;
    s << this->gate->GetName() << this->GetInputs().size();
    string name(s.str());
    transform(name.begin(), name.end(), name.begin(), ::tolower);
    tmp.setGateName(name);
    tmp.setName(this->id);
    
    tmp.setInputs(inputs);
    for(int i = 0; i < inputs; ++i) {
        tmp.setIName(i, this->inputs[i]->id);
    }
    (*pla)[this->id] = tmp;
    (*keys).push_back(this->id);
    return;
}

/**
 * Appends its ID to vector of utilized genes keys and calls self on gene's inputs
 */
void Gene::AppendGeneID_rec(vector<string>* keys) {
    if(!this->gate)
        return;
    if(find(keys->begin(), keys->end(), this->id) != keys->end())
        return;
    this->inPhenotype = true;
    keys->push_back(this->id);
    
    int inputs = this->inputs.size();
    Gene* tmpg;
    for(int j = 0; j < inputs; ++j) {
        tmpg = this->inputs[j];
        tmpg->AppendGeneID_rec(keys);
    }
}

