/* 
 * File:   Gene.h
 * Author: Stanislav Pelák
 */

#ifndef GENE_H
#define	GENE_H

#include "Gate.h"
#include "Kernel/SoPLA.h"
#include <string>
#include <map>
#include <assert.h>

using namespace std;

class Gene {
public:
    Gene(string id, int index);
    Gene(const Gene& orig);
    virtual ~Gene();
    
    void SetGate(Gate* gate) {
        this->gate = gate;
        vector<Gene*> v(gate->GetInputs());
        this->inputs = v;
    }

    void RemoveGate() {
        this->gate = NULL;
        this->inputs.clear();
    }

    void SetInput(int idx, Gene* input)  {
        assert(this->inputs.size() > idx);
        this->inputs[idx] = input;
    }
    
    Gate* GetGate() const { return gate; }
    vector<Gene*> GetInputs() const { return inputs; }
    void addInput(Gene* input) { this->inputs.push_back(input); }
    string GetID() const { return id; }
    void SetId(string id) { this->id = id; }
    int GetIndex() const { return index; }
    void SetIndex(int index) { this->index = index; }
    bool IsInPhenotype() { return this->inPhenotype; }
    bool SetInPhenotype(bool inPhenotype) { this->inPhenotype = inPhenotype; }
    
    void AppendGateCNF_rec(map<string, string>* cnf);
    void AppendGateSoPLA_rec(map<string, SoPLA>* pla, vector<string>* keys);
    void AppendGeneID_rec(vector<string>* keys);

private:
    Gate* gate;
    vector<Gene*> inputs;
    vector<string> test;
    string id;
    int index;
    bool inPhenotype;
};

#endif	/* GENE_H */

